/** Java 1.3 AST Recognizer Grammar
 *
 * Author: (see java.g preamble)
 *
 * This grammar is in the PUBLIC DOMAIN
 */
class JavaDefsTreeParser extends JavaTreeParser;

options {
	importVocab = Java;
}

{
    FileScope fileScope = null;
    java.util.Stack scopeStack = new java.util.Stack();
    CodeProject project = null;

    public void setProject(CodeProject project) {
        this.project = project;
    }

    protected String fileName = null;
    public void setFileName(String f) {
        fileName = f;
    }
}

compilationUnit
{
    fileScope = new FileScope(project,fileName,null);
    scopeStack.push(fileScope);
    project.setFileScope(fileName, fileScope);
    // System.out.println("Define FileScope: "+fileScope);
}
    :
        (packageDefinition)?
		(importDefinition)*
		(typeDefinition)*
	;

packageDefinition
{
    String id = null;
}
	:	#( PACKAGE_DEF id=identifier )
        {
        project.mkPackageScopes(id);
        PackageScope p = project.getPackageScope(id);
        fileScope.setPackageScope(p);
        // System.out.println("Define package: "+p.getFullyQualifiedName());
        }
	;

typeDefinition
	:	#(  CLASS_DEF modifiers IDENT extendsClause implementsClause
            {
            Scope currentScope = (Scope)scopeStack.peek();
            ClassScope cs = new ClassScope(project,#IDENT.getText(),currentScope);
            currentScope.addScope(cs);
            scopeStack.push(cs);
            }
            objBlock
            {scopeStack.pop();}
         )
	|	#(  INTERFACE_DEF modifiers IDENT extendsClause
            {
            Scope currentScope = (Scope)scopeStack.peek();
            ClassScope cs = new ClassScope(project,#IDENT.getText(),currentScope);
            cs.setInterface(true);
            currentScope.addScope(cs);
            scopeStack.push(cs);
            }
            interfaceBlock
            {scopeStack.pop();}
         )
	;

variableDeclarator
	:	IDENT
        {
        Scope currentScope = (Scope)scopeStack.peek();
        if ( currentScope instanceof ClassScope ) {
            FieldVariable fs = new FieldVariable(project,#IDENT.getText(),currentScope);
            currentScope.addScopeMember(fs);
        }
        }
	|	LBRACK variableDeclarator
	;

ctorDef
	:	#(CTOR_DEF modifiers methodHead (slist)?)
        {scopeStack.pop();}
	;

methodDef
	:	#(METHOD_DEF modifiers typeSpec methodHead (slist)?)
        {scopeStack.pop();}
	;

methodHead
	:	IDENT #( PARAMETERS (parameterDef)* ) (throwsClause)?
        {
            Scope currentScope = (Scope)scopeStack.peek();
            MethodScope ms = new MethodScope(project,#IDENT.getText(),currentScope);
            currentScope.addScopeMember(ms);
            project.setMethodScope(ms.getFullyQualifiedName(), ms);
            scopeStack.push(ms);
        }
	;

identifier returns [String id=null]
{
    String r = null;
}
	:	IDENT {id=#IDENT.getText();}
	|	#( DOT r=identifier IDENT ) {id=r+"."+#IDENT.getText();}
	;
